@@ -1,8 +1,18 @@
 # -*- coding: utf-8 -*-
 
+"""
+Diverse color mathematical functions.
+
+Note:
+In most cases, unless otherwise stated RGB is R'G'B' (gamma-compressed)
+
+"""
+
+import colorsys
 import logging
 import math
 import sys
+import warnings
 
 
 def get_transfer_function_phi(alpha, gamma):
@@ -106,7 +116,7 @@ def specialpow(a, b, slope_limit=0):
 		v = (max(a ** (1.0 / SMPTE2084_M2) - SMPTE2084_C1, 0) /
 			 (SMPTE2084_C2 - SMPTE2084_C3 * a ** (1.0 / SMPTE2084_M2))) ** (1.0 / SMPTE2084_M1)
 	else:
-		raise ValueError("Invalid gamma %s" % b)
+		raise ValueError("Invalid gamma %r" % b)
 	return v * signScale
 
 
@@ -145,6 +155,147 @@ def DICOM(j, inverse=False):
 				 h * math.pow(logj, 4) + k * math.pow(logj, 5)))
 
 
+class HLG(object):
+	"""
+	Hybrid Log Gamma (HLG) as defined in Rec BT.2100
+	
+	"""
+
+	def __init__(self, black_cdm2=0.0, white_cdm2=1000.0, system_gamma=1.2,
+				 ambient_cdm2=5, rgb_space="Rec. 2020"):
+		self.black_cdm2 = black_cdm2
+		self.white_cdm2 = white_cdm2
+		self.rgb_space = get_rgb_space(rgb_space)
+		self.system_gamma = system_gamma
+		self.ambient_cdm2 = ambient_cdm2
+
+	@property
+	def gamma(self):
+		""" System gamma for nominal peak luminance and ambient """
+		# Adjust system gamma for peak luminance != 1000 cd/m2
+		gamma = self.system_gamma + 0.42 * math.log10(self.white_cdm2 / 1000.0)
+		if self.ambient_cdm2 > 0:
+			# Adjust system gamma for ambient surround != 5 cd/m2 (BT.2390-3)
+			gamma -= 0.076 * math.log10(self.ambient_cdm2 / 5.0)
+		return gamma
+		
+	def oetf(self, v, inverse=False):
+		"""
+		Hybrid Log Gamma (HLG) OETF
+		
+		Relative scene linear light to non-linear HLG signal, or inverse
+
+		Input domain 0..1
+		Output range 0..1
+		
+		"""
+		if v == 1:
+			return 1.0
+		a = 0.17883277
+		b = 1 - 4 * a
+		c = 0.5 - a * math.log(4 * a)
+		if inverse:
+			# Non-linear HLG signal to relative scene linear light
+			if 0 <= v <= 1 / 2.:
+				v = v ** 2 / 3.
+			else:
+				v = (math.exp((v - c) / a) + b) / 12.
+		else:
+			# Relative scene linear light to non-linear HLG signal
+			if 0 <= v <= 1 / 12.:
+				v = math.sqrt(3 * v)
+			else:
+				v = a * math.log(12 * v - b) + c
+		return v
+
+	def eotf(self, RGB, inverse=False, apply_black_offset=True):
+		"""
+		Hybrid Log Gamma (HLG) EOTF
+		
+		Non-linear HLG signal to display light, or inverse
+		
+		Input domain 0..1
+		Output range 0..1
+		
+		"""
+		if isinstance(RGB, (float, int)):
+			R, G, B = (RGB,) * 3
+		else:
+			R, G, B = RGB
+		if inverse:
+			# Display light -> relative scene linear light -> HLG signal
+			R, G, B = (self.oetf(v) for v in self.ootf((R, G, B), True,
+													   apply_black_offset))
+		else:
+			# HLG signal -> relative scene linear light -> display light
+			R, G, B = self.ootf([self.oetf(v, True) for v in (R, G, B)], False,
+								apply_black_offset)
+		return G if isinstance(RGB, (float, int)) else (R, G, B)
+		
+
+	def ootf(self, RGB, inverse=False, apply_black_offset=True):
+		"""
+		Hybrid Log Gamma (HLG) OOTF
+
+		Relative scene linear light to display light, or inverse
+		
+		Input domain 0..1
+		Output range 0..1
+		
+		"""
+		if isinstance(RGB, (float, int)):
+			R, G, B = (RGB,) * 3
+		else:
+			R, G, B = RGB
+		if apply_black_offset:
+			black_cdm2 = float(self.black_cdm2)
+		else:
+			black_cdm2 = 0
+		alpha = (self.white_cdm2 - black_cdm2) / self.white_cdm2
+		beta = black_cdm2 / self.white_cdm2
+		Y = 0.2627 * R + 0.6780 * G + 0.0593 * B
+		if inverse:
+			if Y > beta:
+				R, G, B = (((Y - beta) / alpha) ** ((1 - self.gamma) / self.gamma) *
+						   ((v - beta) / alpha) for v in (R, G, B))
+			else:
+				R, G, B = 0, 0, 0
+		else:
+			if Y:
+				Y **= (self.gamma - 1)
+			R, G, B = (alpha * Y * E + beta for E in (R, G, B))
+		return G if isinstance(RGB, (float, int)) else (R, G, B)
+
+	def RGB2XYZ(self, R, G, B, apply_black_offset=True):
+		""" Non-linear HLG signal to display XYZ """
+		X, Y, Z = self.rgb_space[-1] * [self.oetf(v, True) for v in (R, G, B)]
+		X, Y, Z = (max(v, 0) for v in (X, Y, Z))
+		Yy = self.ootf(Y, apply_black_offset=False)
+		if Y:
+			X, Y, Z = (v / Y * Yy for v in (X, Y, Z))
+		else:
+			X, Y, Z = (v * Yy for v in self.rgb_space[1])
+		if apply_black_offset:
+			beta = self.ootf(0)
+			bp_out = [v * beta for v in self.rgb_space[1]]
+			X, Y, Z = apply_bpc(X, Y, Z, (0, 0, 0), bp_out, self.rgb_space[1])
+		return X, Y, Z
+
+	def XYZ2RGB(self, X, Y, Z, apply_black_offset=True):
+		""" Display XYZ to non-linear HLG signal """
+		if apply_black_offset:
+			beta = self.ootf(0)
+			bp_in = [v * beta for v in self.rgb_space[1]]
+			X, Y, Z = apply_bpc(X, Y, Z, bp_in, (0, 0, 0), self.rgb_space[1])
+		Yy = self.ootf(Y, True, apply_black_offset=False)
+		if Y:
+			X, Y, Z = (v / Y * Yy for v in (X, Y, Z))
+		R, G, B = self.rgb_space[-1].inverted() * (X, Y, Z)
+		R, G, B = (max(v, 0) for v in (R, G, B))
+		R, G, B = [self.oetf(v) for v in (R, G, B)]
+		return R, G, B
+
+
 rgb_spaces = {
 	# http://brucelindbloom.com/WorkingSpaceInfo.html
 	# ACES: https://github.com/ampas/aces-dev/blob/master/docs/ACES_1.0.1.pdf?raw=true
@@ -156,6 +307,7 @@ rgb_spaces = {
 	# name              gamma             white                     primaries
 	#                                     point                     Rx      Ry      RY          Gx      Gy      GY          Bx      By      BY
 	"ACES":             (1.0,             (0.95265, 1.0, 1.00883), (0.7347, 0.2653, 0.343961), (0.0000, 1.0000, 0.728164), (0.0001,-0.0770,-0.072125)),
+	"ACEScg":           (1.0,             (0.95265, 1.0, 1.00883), (0.7130, 0.2930, 0.272230), (0.1650, 0.8300, 0.674080), (0.1280, 0.0440, 0.053690)),
 	"Adobe RGB (1998)": (2 + 51 / 256.0,  "D65",                   (0.6400, 0.3300, 0.297361), (0.2100, 0.7100, 0.627355), (0.1500, 0.0600, 0.075285)),
 	"Apple RGB":        (1.8,             "D65",                   (0.6250, 0.3400, 0.244634), (0.2800, 0.5950, 0.672034), (0.1550, 0.0700, 0.083332)),
 	"Best RGB":         (2.2,             "D50",                   (0.7347, 0.2653, 0.228457), (0.2150, 0.7750, 0.737352), (0.1300, 0.0350, 0.034191)),
@@ -163,6 +315,7 @@ rgb_spaces = {
 	"Bruce RGB":        (2.2,             "D65",                   (0.6400, 0.3300, 0.240995), (0.2800, 0.6500, 0.683554), (0.1500, 0.0600, 0.075452)),
 	"CIE RGB":          (2.2,             "E",                     (0.7350, 0.2650, 0.176204), (0.2740, 0.7170, 0.812985), (0.1670, 0.0090, 0.010811)),
 	"ColorMatch RGB":   (1.8,             "D50",                   (0.6300, 0.3400, 0.274884), (0.2950, 0.6050, 0.658132), (0.1500, 0.0750, 0.066985)),
+#	"DCDM X'Y'Z'":      (2.6,             "E",                     (1.0000, 0.0000, 0.000000), (0.0000, 1.0000, 1.000000), (0.0000, 0.0000, 0.000000)),
 	"DCI P3":           (2.6,             (0.89459, 1.0, 0.95442), (0.6800, 0.3200, 0.209475), (0.2650, 0.6900, 0.721592), (0.1500, 0.0600, 0.068903)),
 	"Don RGB 4":        (2.2,             "D50",                   (0.6960, 0.3000, 0.278350), (0.2150, 0.7650, 0.687970), (0.1300, 0.0350, 0.033680)),
 	"ECI RGB":          (1.8,             "D50",                   (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
@@ -173,7 +326,6 @@ rgb_spaces = {
 	"ProPhoto RGB":     (1.8,             "D50",                   (0.7347, 0.2653, 0.288040), (0.1596, 0.8404, 0.711874), (0.0366, 0.0001, 0.000086)),
 	"Rec. 709":         (-709,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
 	"Rec. 2020":        (-709,            "D65",                   (0.7080, 0.2920, 0.262694), (0.1700, 0.7970, 0.678009), (0.1310, 0.0460, 0.059297)),
-	"Rec. 2020 ST2084": (-2084,           "D65",                   (0.7080, 0.2920, 0.262694), (0.1700, 0.7970, 0.678009), (0.1310, 0.0460, 0.059297)),
 	"SMPTE-C":          (2.2,             "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
 	"SMPTE 240M":       (-240,            "D65",                   (0.6300, 0.3400, 0.212395), (0.3100, 0.5950, 0.701049), (0.1550, 0.0700, 0.086556)),
 	"sRGB":             (-2.4,            "D65",                   (0.6400, 0.3300, 0.212656), (0.3000, 0.6000, 0.715158), (0.1500, 0.0600, 0.072186)),
@@ -229,10 +381,10 @@ def LMS_wp_adaption_matrix(whitepoint_source=None,
 	XYZWD = get_whitepoint(whitepoint_destination)
 	if XYZWS[1] <= 1.0 and XYZWD[1] > 1.0:
 		# make sure the scaling is identical
-		XYZWS = [v * 100 for v in XYZWS]
+		XYZWD = [v / XYZWD[1] * XYZWS[1] for v in XYZWD]
 	if XYZWD[1] <= 1.0 and XYZWS[1] > 1.0:
 		# make sure the scaling is identical
-		XYZWD = [v * 100 for v in XYZWD]
+		XYZWS = [v / XYZWS[1] * XYZWD[1] for v in XYZWS]
 	Ls, Ms, Ss = XYZ2LMS(XYZWS[0], XYZWS[1], XYZWS[2], cat)
 	Ld, Md, Sd = XYZ2LMS(XYZWD[0], XYZWD[1], XYZWD[2], cat)
 	return Matrix3x3([[Ld/Ls, 0, 0], [0, Md/Ms, 0], [0, 0, Sd/Ss]])
@@ -308,7 +460,7 @@ def blend_blackpoint(X, Y, Z, bp_in=None, bp_out=None, power=40.0):
 	L, a, b = XYZ2Lab(*[v * 100 for v in (X, Y, Z)])
 
 	for bp in (bp_in, bp_out):
-		if not bp:
+		if not bp or tuple(bp) == (0, 0, 0):
 			continue
 		bpLab = XYZ2Lab(*[v * 100 for v in bp])
 		if bp is bp_in:
@@ -374,6 +526,49 @@ def interp(x, xp, fp, left=None, right=None):
 		return fp[lower] + (fp[higher] - fp[lower]) / steps
 
 
+def interp_resize(iterable, new_size, use_numpy=False):
+	""" Change size of iterable through linear interpolation """
+	result = []
+	x_new = range(len(iterable))
+	interp = Interp(x_new, iterable, use_numpy=use_numpy)
+	for i in xrange(new_size):
+		result.append(interp(i / (new_size - 1.0) * (len(iterable) - 1.0)))
+	return result
+
+
+def smooth_avg(values, passes=1, window=None):
+	"""
+	Smooth values (moving average).
+	
+	passses   Number of passes
+	window    Tuple or list containing weighting factors. Its length
+			  determines the size of the window to use.
+			  Defaults to (1.0, 1.0, 1.0)
+	
+	"""
+	if not window or len(window) < 3 or len(window) % 2 != 1:
+		window = (1.0, 1.0, 1.0)
+	for x in xrange(0, passes):
+		data = []
+		for j, v in enumerate(values):
+			tmpwindow = window
+			while j > 0 and j < len(values) - 1 and len(tmpwindow) >= 3:
+				tl = (len(tmpwindow) - 1) / 2
+				# print j, tl, tmpwindow
+				if tl > 0 and j - tl >= 0 and j + tl <= len(values) - 1:
+					windowslice = values[j - tl:j + tl + 1]
+					windowsize = 0
+					for k, weight in enumerate(tmpwindow):
+						windowsize += float(weight) * windowslice[k]
+					v = windowsize / sum(tmpwindow)
+					break
+				else:
+					tmpwindow = tmpwindow[1:-1]
+			data.append(v)
+		values = data
+	return data
+
+
 def compute_bpc(bp_in, bp_out):
 	"""
 	Black point compensation. Implemented as a linear scaling in XYZ. 
@@ -410,7 +605,8 @@ def compute_bpc(bp_in, bp_out):
 	return matrix, offset
 
 
-def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
+def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None,
+		  cie94_use_symmetric_chrominance=True):
 		"""
 		Compute the delta of two samples
 
@@ -454,11 +650,10 @@ def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
 			SL = 1.0
 			K1 = 0.048 if textiles else 0.045
 			K2 = 0.014 if textiles else 0.015
-			# brucelindbloom.com formula originally used C1 instead of C_,
-			# but the results are different from ProfileMaker/MeasureTool 
-			# implementation, so use this instead
-			# (found on www.farbmetrik-gall.de/cielab/korrcielab/cie94.html)
-			C_ = (math.sqrt((1 + (K1 * C1)) * (1 + (K1 * C2))) - 1) / K1
+			if cie94_use_symmetric_chrominance:
+				C_ = math.sqrt(C1 * C2)
+			else:
+				C_ = C1
 			SC = 1.0 + K1 * C_
 			SH = 1.0 + K2 * C_
 			KL = 2.0 if textiles else 1.0
@@ -551,6 +746,43 @@ def is_similar_matrix(matrix1, matrix2, digits=3):
 	return matrix1.rounded(digits) == matrix2.rounded(digits)
 
 
+def four_color_matrix(XrR, YrR, ZrR, XrG, YrG, ZrG, XrB, YrB, ZrB, XrW, YrW, ZrW,
+					  XmR, YmR, ZmR, XmG, YmG, ZmG, XmB, YmB, ZmB, XmW, YmW, ZmW,
+					  Y_correction=True):
+	"""
+	Four-Color Matrix Method for Correction of Tristimulus Colorimeters
+	
+	Based on paper published in Proc., IS&T Fifth Color Imaging Conference,
+	301-305 (1997) and IS&T Sixth Color Imaging Conference (1998).
+	
+	"""
+	XYZ = locals()
+	xyz = {}
+	M = {}
+	k = {}
+	for s in "mr":
+		xyz[s] = {}
+		for color in "RGBW":
+			X, Y, Z = (XYZ[component + s + color] for component in "XYZ")
+			x, y = XYZ2xyY(X, Y, Z)[:2]
+			xyz[s][color] = x, y, 1 - x - y
+		M[s] = Matrix3x3([xyz[s][color] for color in "RGB"]).transposed()
+		k[s] = M[s].inverted() * xyz[s]["W"]
+		M[s + "RGB"] = M[s] * Matrix3x3([[k[s][0], 0, 0],
+										 [0, k[s][1], 0],
+										 [0, 0, k[s][2]]])
+	R = M["rRGB"] * M["mRGB"].inverted()
+	if Y_correction:
+		# The Y calibration factor kY is obtained as the ratio of the reference
+		# luminance value to the matrix-corrected Y value, as defined in
+		# Four-Color Matrix Method for Correction of Tristimulus Colorimeters –
+		# Part 2
+		MW = XmW, YmW, ZmW
+		kY = YrW / (R * MW)[1]
+		R[:] = [[kY * v for v in row] for row in R]
+	return R
+
+
 def get_gamma(values, scale=1.0, vmin=0.0, vmax=1.0, average=True, least_squares=False):
 	""" Return average or least squares gamma or a list of gamma values """
 	if least_squares:
@@ -586,7 +818,7 @@ def guess_cat(chad, whitepoint_source=None, whitepoint_destination=None):
 	""" Try and guess the chromatic adaption transform used in a chromatic 
 	adaption matrix as found in an ICC profile's 'chad' tag """
 	if chad == [[1, 0, 0], [0, 1, 0], [0, 0, 1]]:
-		return "None"
+		return "XYZ scaling"
 	for cat in cat_matrices:
 		if is_similar_matrix((chad * cat_matrices[cat].inverted() * 
 							  LMS_wp_adaption_matrix(whitepoint_destination, 
@@ -677,12 +909,18 @@ def DIN99bcdLCH2Lab(L99, C99, H99, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
 	H99 -= deg
 	L, a, b = DIN99familyLHCG2Lab(L99, H99, C99, G, 1.0, l1, l2, deg, f1)
 	if x:
-		X, Y, Z = Lab2XYZ(L, a, b, whitepoint, scale=100)
-		X = (X + x * Z) / (1 + x)
+		whitepoint99d = XYZ2DIN99cdXYZ(*get_whitepoint(whitepoint, 100), x=x)
+		X, Y, Z = Lab2XYZ(L, a, b, whitepoint99d, scale=100)
+		X, Y, Z = DIN99cdXYZ2XYZ(X, Y, Z, x)
 		L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
 	return L, a, b
 
 
+def DIN99cdXYZ2XYZ(X, Y, Z, x):
+	X = (X + x * Z) / (1 + x)
+	return X, Y, Z
+
+
 def DIN99familyLHCG2Lab(L99, H99, C99, G, kE, l1, l2, deg, f1):
 	L = (math.exp((L99 * kE) / l1) - 1) / l2
 	h99ef = H99 * math.pi / 180
@@ -718,42 +956,12 @@ def DIN99familyab2DIN99CH(a99, b99):
 	return C99, H99
 
 
+def HSL2RGB(H, S, L, scale=1.0):
+    return tuple(v * scale for v in colorsys.hls_to_rgb(H, L, S))
+
+
 def HSV2RGB(H, S, V, scale=1.0):
-	if S == 0:
-		return (V * scale, ) * 3
-	else:
-		h = H * 6.0
-		if h == 6:
-			h = 0
-		i = int(h)
-		component_1 = V * (1.0 - S)
-		component_2 = V * (1.0 - S * (h - i))
-		component_3 = V * (1.0 - S * (1.0 - (h - i)))
-		if i == 0:
-			R = V
-			G = component_3
-			B = component_1
-		elif i == 1:
-			R = component_2
-			G = V
-			B = component_1
-		elif i == 2:
-			R = component_1
-			G = V
-			B = component_3
-		elif i == 3:
-			R = component_1
-			G = component_2
-			B = V
-		elif i == 4:
-			R = component_3
-			G = component_1
-			B = V
-		else:
-			R = V
-			G = component_1
-			B = component_2
-		return R * scale, G * scale, B * scale
+	return tuple(v * scale for v in colorsys.hsv_to_rgb(H, S, V))
 
 
 def get_DBL_MIN():
@@ -858,12 +1066,13 @@ def Lab2Luv(L, a, b, whitepoint=None, scale=100):
 
 
 def Lab2RGB(L, a, b, rgb_space=None, scale=1.0, round_=False, clamp=True,
-			whitepoint=None, noadapt=False, cat="Bradford"):
+			whitepoint=None, whitepoint_source=None, noadapt=False,
+			cat="Bradford"):
 	""" Convert from Lab to RGB """
 	X, Y, Z = Lab2XYZ(L, a, b, whitepoint)
 	if not noadapt:
-		X, Y, Z = adapt(X, Y, Z, whitepoint, rgb_space[1] if rgb_space
-											 else "D65", cat)
+		X, Y, Z = adapt(X, Y, Z, whitepoint_source, rgb_space[1] if rgb_space
+													else "D65", cat)
 	return XYZ2RGB(X, Y, Z, rgb_space, scale, round_, clamp)
 
 
@@ -977,99 +1186,42 @@ def RGB2HSI(R, G, B, scale=1.0):
 
 
 def RGB2HSL(R, G, B, scale=1.0):
-	RGB_min = min(R, G, B)
-	RGB_max = max(R, G, B)
-	delta = RGB_max - RGB_min
-	L = (RGB_max + RGB_min) / 2.0
-	if delta == 0:
-		# Gray
-		H = 0
-		S = 0
-	else:
-		if L < .5:
-			S = delta / (RGB_max + RGB_min)
-		else:
-			S = delta / (2.0 - RGB_max - RGB_min)
-		delta_R = (((RGB_max - R) / 6.0) + (delta / 2.0) ) / delta
-		delta_G = (((RGB_max - G) / 6.0) + (delta / 2.0) ) / delta
-		delta_B = (((RGB_max - B) / 6.0) + (delta / 2.0) ) / delta
-		if R == RGB_max:
-			H = delta_B - delta_G
-		elif G == RGB_max:
-			H = (1.0 / 3.0) + delta_R - delta_B
-		elif B == RGB_max:
-			H = (2.0 / 3.0) + delta_G - delta_R
-		if H < 0:
-			H += 1.0
-		if H > 1:
-			H -= 1.0
-	return H * scale, S * scale, L * scale
+	H, L, S = colorsys.rgb_to_hls(R, G, B)
+	return tuple(v * scale for v in (H, S, L))
 
 
 def RGB2HSV(R, G, B, scale=1.0):
-	RGB_min = min( R, G, B )
-	RGB_max = max( R, G, B )
-	delta = RGB_max - RGB_min
-	V = RGB_max
-	if ( delta == 0 ):
-		# Gray
-		H = 0
-		S = 0
-	else:
-		S = delta / RGB_max
-		delta_R = (((RGB_max - R) / 6.0) + (delta / 2.0) ) / delta
-		delta_G = (((RGB_max - G) / 6.0) + (delta / 2.0) ) / delta
-		delta_B = (((RGB_max - B) / 6.0) + (delta / 2.0) ) / delta
-		if R == RGB_max:
-			H = delta_B - delta_G
-		elif G == RGB_max:
-			H = (1.0 / 3.0) + delta_R - delta_B
-		elif B == RGB_max:
-			H = (2.0 / 3.0) + delta_G - delta_R
-		if H < 0:
-			H += 1.0
-		if H > 1:
-			H -= 1.0
-	return H * scale, S * scale, V * scale
+	return tuple(v * scale for v in colorsys.rgb_to_hsv(R, G, B))
 
 
-def RGB2ICtCp(R, G, B, rgb_space="Rec. 2020 ST2084"):
+def LinearRGB2ICtCp(R, G, B, oetf=lambda FD: specialpow(FD, 1.0 / -2084)):
+	""" Rec. 2020 linear RGB to non-linear ICtCp """
 	# http://www.dolby.com/us/en/technologies/dolby-vision/ICtCp-white-paper.pdf
-	rgb2lms_matrix = Matrix3x3([[1688 / 4096., 2146 / 4096., 262 / 4096.],
-								[683 / 4096., 2951 / 4096., 462 / 4096.],
-								[99 / 4096., 309 / 4096., 3688 / 4096.]])
-	LMS = rgb2lms_matrix * (R, G, B)
-	L_, M_, S_ = (specialpow(FD, 1.0 / -2084) for FD in LMS)
-	L_M_S_2ICtCp_matrix = Matrix3x3([[.5, .5, 0],
-									 [6610 / 4096., -13613 / 4096., 7003 / 4096.],
-									 [17933 / 4096., -17390 / 4096., -543 / 4096.]])
+	LMS = LinearRGB2LMS_matrix * (R, G, B)
+	L_, M_, S_ = (oetf(FD) for FD in LMS)
 	I, Ct, Cp = L_M_S_2ICtCp_matrix * (L_, M_, S_)
 	return I, Ct, Cp
 
 
-def ICtCp2RGB(I, Ct, Cp, rgb_space="Rec. 2020 ST2084"):
+def ICtCp2LinearRGB(I, Ct, Cp, eotf=lambda v: specialpow(v, -2084)):
+	""" Non-linear ICtCp to Rec. 2020 linear RGB """
 	# http://www.dolby.com/us/en/technologies/dolby-vision/ICtCp-white-paper.pdf	
-	L_M_S_2ICtCp_matrix = Matrix3x3([[.5, .5, 0],
-									 [6610 / 4096., -13613 / 4096., 7003 / 4096.],
-									 [17933 / 4096., -17390 / 4096., -543 / 4096.]])
-	L_M_S_ = L_M_S_2ICtCp_matrix.inverted() * (I, Ct, Cp)
-	L, M, S = (specialpow(v, -2084) for v in L_M_S_)
-	rgb2lms_matrix = Matrix3x3([[1688 / 4096., 2146 / 4096., 262 / 4096.],
-								[683 / 4096., 2951 / 4096., 462 / 4096.],
-								[99 / 4096., 309 / 4096., 3688 / 4096.]])
-	R, G, B = rgb2lms_matrix.inverted() * (L, M, S)
+	L_M_S_ = ICtCp2L_M_S__matrix * (I, Ct, Cp)
+	L, M, S = (eotf(v) for v in L_M_S_)
+	R, G, B = LMS2LinearRGB_matrix * (L, M, S)
 	return R, G, B
 
 
-def XYZ2ICtCp(X, Y, Z, rgb_space="Rec. 2020 ST2084", clamp=False):
-	R, G, B = (specialpow(v, -2084) for v in XYZ2RGB(X, Y, Z, rgb_space,
-													 clamp=clamp))
-	return RGB2ICtCp(R, G, B, rgb_space)
+def XYZ2ICtCp(X, Y, Z, rgb_space="Rec. 2020", clamp=False,
+			  oetf=lambda E: specialpow(E, 1.0 / -2084)):
+	R, G, B = XYZ2RGB(X, Y, Z, rgb_space, clamp=clamp, oetf=lambda v: v)
+	return LinearRGB2ICtCp(R, G, B, oetf)
 
 
-def ICtCp2XYZ(I, Ct, Cp, rgb_space="Rec. 2020 ST2084"):
-	R, G, B = (specialpow(v, 1.0 / -2084) for v in ICtCp2RGB(I, Ct, Cp, rgb_space))
-	return RGB2XYZ(R, G, B, rgb_space)
+def ICtCp2XYZ(I, Ct, Cp, rgb_space="Rec. 2020",
+			  eotf=lambda v: specialpow(v, -2084)):
+	R, G, B = ICtCp2LinearRGB(I, Ct, Cp, eotf)
+	return RGB2XYZ(R, G, B, rgb_space, eotf=lambda v: v)
 
 
 def RGB2Lab(R, G, B, rgb_space=None, whitepoint=None, noadapt=False,
@@ -1081,7 +1233,7 @@ def RGB2Lab(R, G, B, rgb_space=None, whitepoint=None, noadapt=False,
 	return XYZ2Lab(X, Y, Z, whitepoint=whitepoint)
 
 
-def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
+def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0, eotf=None):
 	"""
 	Convert from RGB to XYZ.
 	
@@ -1128,7 +1280,9 @@ def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
 			gamma = trc[i]
 		else:
 			gamma = trc
-		if isinstance(gamma, (list, tuple)):
+		if eotf:
+			RGB[i] = eotf(v)
+		elif isinstance(gamma, (list, tuple)):
 			RGB[i] = interp(v, [n / float(len(gamma) - 1) for n in
 							    xrange(len(gamma))], gamma)
 		else:
@@ -1137,6 +1291,83 @@ def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
 	return tuple(v * scale for v in XYZ)
 
 
+def RGB2YCbCr(R, G, B, rgb_space="NTSC 1953", bits=8):
+	""" R'G'B' to Y'CbCr quantized to n bits """
+	return YPbPr2YCbCr(*RGB2YPbPr(R, G, B, rgb_space), bits=bits)
+
+
+def RGB2YPbPr(R, G, B, rgb_space="NTSC 1953"):
+	""" R'G'B' to Y'PbPr """
+	return RGB2YPbPr_matrix(rgb_space) * (R, G, B)
+
+
+def RGB2YPbPr_matrix(rgb_space="NTSC 1953"):
+	(trc, whitepoint, (rx, ry, rY), (gx, gy, gY), (bx, by, bY),
+	 matrix) = get_rgb_space(rgb_space)
+	if matrix == get_rgb_space("NTSC 1953")[-1]:
+		ndigits = 3
+	else:
+		ndigits = 4
+	KR = round(rY, ndigits)
+	KB = round(bY, ndigits)
+	KG = 1.0 - KR - KB
+	Pb_scale = ((1 - KB) / 0.5)
+	Pr_scale = ((1 - KR) / 0.5)
+	return Matrix3x3([[KR, KG, KB],
+					  [-KR / Pb_scale, -KG / Pb_scale, 0.5],
+					  [0.5, -KG / Pr_scale, -KB / Pr_scale]])
+
+
+def YCbCr2YPbPr(Y, Cb, Cr, bits=8):
+	""" Y'CbCr to Y'PbPr """
+	bitlevels = 2 ** bits
+	Yblack = 16 / 256.0 * bitlevels
+	Yscale = 219 / 256.0 * bitlevels
+	Y -= Yblack
+	Y /= Yscale
+	Cneutral = 128 / 256.0 * bitlevels
+	Cscale = 224 / 256.0 * bitlevels
+	Cb -= Cneutral
+	Cb /= Cscale
+	Cr -= Cneutral
+	Cr /= Cscale
+	return Y, Cb, Cr
+
+
+def YCbCr2RGB(Y, Cb, Cr, rgb_space="NTSC 1953", bits=8, scale=1.0, round_=False,
+			  clamp=True):
+	""" Y'CbCr to R'G'B' """
+	Y, Pb, Pr = YCbCr2YPbPr(Y, Cb, Cr, bits)
+	return YPbPr2RGB(Y, Pb, Pr, rgb_space, scale, round_, clamp)
+
+
+def YPbPr2RGB(Y, Pb, Pr, rgb_space="NTSC 1953", scale=1.0, round_=False,
+			  clamp=True):
+	""" Y'PbPr to R'G'B' """
+	RGB = RGB2YPbPr_matrix(rgb_space).inverted() * (Y, Pb, Pr)
+	for i in xrange(3):
+		if clamp:
+			RGB[i] = min(1.0, max(0.0, RGB[i]))
+		RGB[i] *= scale
+		if round_ is not False:
+			RGB[i] = round(RGB[i], round_)
+	return RGB
+
+
+def YPbPr2YCbCr(Y, Pb, Pr, bits=8):
+	""" Y'PbPr' to Y'CbCr quantized to n bits """
+	bitlevels = 2 ** bits
+	Yblack = 16 / 256.0 * bitlevels
+	Yscale = 219 / 256.0 * bitlevels
+	Y = Yblack + Yscale * Y
+	Cneutral = 128 / 256.0 * bitlevels
+	Cscale = 224 / 256.0 * bitlevels
+	Cb = Cneutral + Cscale * Pb
+	Cr = Cneutral + Cscale * Pr
+	Y, Cb, Cr = (int(round(v)) for v in (Y, Cb, Cr))
+	return Y, Cb, Cr
+
+
 def RGBsaturation(R, G, B, saturation, rgb_space=None):
 	""" (De)saturate a RGB color in CIE xy and return the RGB and xyY values """
 	whitepoint = RGB2XYZ(1, 1, 1, rgb_space=rgb_space)
@@ -1158,6 +1389,12 @@ def xyYsaturation(x, y, Y, wx, wy, saturation):
 	return wx + (x - wx) * saturation,  wy + (y - wy) * saturation, Y
 
 
+def convert_range(v, oldmin=0, oldmax=1, newmin=0, newmax=1):
+	oldrange = float(oldmax - oldmin)
+	newrange = newmax - newmin
+	return (((v - oldmin) * newrange) / oldrange) + newmin
+
+
 def rgb_to_xyz_matrix(rx, ry, gx, gy, bx, by, whitepoint=None, scale=1.0):
 	""" Create and return an RGB to XYZ matrix. """
 	whitepoint = get_whitepoint(whitepoint, scale)
@@ -1178,7 +1415,7 @@ def get_rgb_space(rgb_space=None, scale=1.0):
 		rgb_space = "sRGB"
 	if isinstance(rgb_space, basestring):
 		rgb_space = rgb_spaces[rgb_space]
-	cachehash = id(rgb_space), scale
+	cachehash = tuple(map(id, rgb_space[:5])), scale
 	cache = get_rgb_space.cache.get(cachehash, None)
 	if cache:
 		return cache
@@ -1221,24 +1458,32 @@ def get_standard_illuminant(illuminant_name="D50",
 get_standard_illuminant.cache = {}
 
 
-def get_whitepoint(whitepoint=None, scale=1.0):
+def get_whitepoint(whitepoint=None, scale=1.0, planckian=False):
 	""" Return a whitepoint as XYZ coordinates """
 	if isinstance(whitepoint, (list, tuple)):
 		return whitepoint
 	if not whitepoint:
 		whitepoint = "D50"
-	cachehash = whitepoint, scale
+	cachehash = whitepoint, scale, planckian
 	cache = get_whitepoint.cache.get(cachehash, None)
 	if cache:
 		return cache
 	if isinstance(whitepoint, basestring):
 		whitepoint = get_standard_illuminant(whitepoint)
 	elif isinstance(whitepoint, (float, int)):
-		whitepoint = CIEDCCT2XYZ(whitepoint)
+		cct = whitepoint
+		if planckian:
+			whitepoint = planckianCT2XYZ(cct)
+			if not whitepoint:
+				raise ValueError("Planckian color temperature %i out of range "
+								 "(1667, 25000)" % cct)
+		else:
+			whitepoint = CIEDCCT2XYZ(cct)
+			if not whitepoint:
+				raise ValueError("Daylight color temperature %i out of range "
+								 "(4000, 25000)" % cct)
 	if scale > 1.0 and whitepoint[1] == 100:
 		scale = 1.0
-	if whitepoint[1] * scale > 100:
-		ValueError("Y value out of range after scaling: %s" % (whitepoint[1] * scale))
 	whitepoint = tuple(v * scale for v in whitepoint)
 	get_whitepoint.cache[cachehash] = whitepoint
 	return whitepoint
@@ -1247,6 +1492,49 @@ def get_whitepoint(whitepoint=None, scale=1.0):
 get_whitepoint.cache = {}
 
 
+def make_monotonically_increasing(iterable, passes=0, window=None):
+	"""
+	Given an iterable or sequence, make the values strictly monotonically
+	increasing (no repeated successive values) by linear interpolation.
+	
+	If iterable is a dict, keep the keys of the original.
+	
+	If passes is non-zero, apply moving average smoothing to the values
+	before making them monotonically increasing.
+	
+	"""
+	if isinstance(iterable, dict):
+		keys = iterable.keys()
+		values = iterable.values()
+	else:
+		if hasattr(iterable, "next"):
+			values = list(iterable)
+		else:
+			values = iterable
+		keys = xrange(len(values))
+	if passes:
+		values = smooth_avg(values, passes, window)
+	sequence = zip(keys, values)
+	numvalues = len(sequence)
+	s_new = []
+	y_min = sequence[0][1]
+	while sequence:
+		x, y = sequence.pop()
+		if (not s_new or y < s_new[0][1]) and (y > y_min or not sequence):
+			s_new.insert(0, (x, y))
+	sequence = s_new
+	# Interpolate to original size
+	x_new = [item[0] for item in sequence]
+	y = [item[1] for item in sequence]
+	values = []
+	for i in xrange(numvalues):
+		values.append(interp(i, x_new, y))
+	if isinstance(iterable, dict):
+		# Add in original keys
+		return iterable.__class__(zip(keys, values))
+	return values
+
+
 def planckianCT2XYZ(T, scale=1.0):
 	""" Convert from planckian temperature to XYZ.
 	
@@ -1331,8 +1619,8 @@ def xyY2XYZ(x, y, Y=1.0):
 	"""
 	if y == 0:
 		return 0, 0, 0
-	X = (x * Y) / y
-	Z = ((1 - x - y) * Y) / y
+	X = float(x * Y) / y
+	Z = float((1 - x - y) * Y) / y
 	return X, Y, Z
 
 
@@ -1461,11 +1749,17 @@ def XYZ2DIN99cd(X, Y, Z, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
 
 
 def XYZ2DIN99cdLCH(X, Y, Z, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
-	X = (1 + x) * X - x * Z
-	L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	X, Y, Z = XYZ2DIN99cdXYZ(X, Y, Z, x)
+	whitepoint99d = XYZ2DIN99cdXYZ(*get_whitepoint(whitepoint, 100), x=x)
+	L, a, b = XYZ2Lab(X, Y, Z, whitepoint99d)
 	return Lab2DIN99bcdLCH(L, a, b, l1, l2, deg, f1, c1, c2)
 
 
+def XYZ2DIN99cdXYZ(X, Y, Z, x):
+	X = (1 + x) * X - x * Z
+	return X, Y, Z
+
+
 def XYZ2DIN99d(X, Y, Z, whitepoint=None):
 	return XYZ2DIN99cd(X, Y, Z, .12, 325.221, .0036, 50, 1.14, 22.5, .06,
 					   whitepoint)
@@ -1476,6 +1770,29 @@ def XYZ2DIN99dLCH(X, Y, Z, whitepoint=None):
 						  whitepoint)
 
 
+def XYZ2IPT(X, Y, Z):
+	XYZ2LMS_matrix = get_cat_matrix("IPT")
+	LMS = XYZ2LMS_matrix * (X, Y, Z)
+	for i, component in enumerate(LMS):
+		if component >= 0:
+			LMS[i] **= 0.43
+		else:
+			LMS[i] = -(-component) ** 0.43
+	return LMS2IPT_matrix * LMS
+
+
+def IPT2XYZ(I, P, T):
+	XYZ2LMS_matrix = get_cat_matrix("IPT")
+	LMS2XYZ_matrix = XYZ2LMS_matrix.inverted()
+	LMS = IPT2LMS_matrix * (I, P, T)
+	for i, component in enumerate(LMS):
+		if component >= 0:
+			LMS[i] **= 1 / 0.43
+		else:
+			LMS[i] = -(-component) ** (1 / 0.43)
+	return LMS2XYZ_matrix * LMS
+
+
 def XYZ2Lab(X, Y, Z, whitepoint=None):
 	"""
 	Convert from XYZ to Lab.
@@ -1505,6 +1822,92 @@ def XYZ2Lab(X, Y, Z, whitepoint=None):
 	return L, a, b
 
 
+def XYZ2Lpt(X, Y, Z, whitepoint=None):
+	"""
+	Convert from XYZ to Lpt
+	
+	This is a modern update to L*a*b*, based on IPT space.
+	
+	Differences to L*a*b* and IPT:
+	- Using inverse CIE 2012 2degree LMS to XYZ matrix instead of
+	  Hunt-Pointer-Estevez Von Kries chromatic adapation in LMS space.
+	- Using L* compression rather than IPT pure 0.43 power.
+	- Tweaked LMS' to IPT matrix to account for change in XYZ to LMS matrix.
+	- Output scaled to L*a*b* type ranges, to maintain 1 JND scale.
+	- L* value is not a non-linear Y value.
+	
+	The input Y value needs to be in the nominal range [0.0, 100.0] and 
+	other input values scaled accordingly.
+	The output L value is in the nominal range [0.0, 100.0].
+	
+	whitepoint can be string (e.g. "D50"), a tuple of XYZ coordinates or 
+	color temperature as float or int. Defaults to D50 if not set.
+	
+	"""
+	# Adapted from Argyll/icc/icc.c
+
+	xyz2lms = get_cat_matrix("CIE2012_2")
+	
+	wlms = xyz2lms * get_whitepoint(whitepoint, 100)
+
+	lms = xyz2lms * (X, Y, Z)
+
+	for j in xrange(3):
+		lms[j] /= wlms[j]
+
+		if (lms[j] > 0.008856451586):
+			lms[j] = pow(lms[j], 1.0 / 3.0);
+		else:
+			lms[j] = 7.787036979 * lms[j] + 16.0 / 116.0
+		lms[j] = 116.0 * lms[j] - 16.0
+
+	return LMS2Lpt_matrix * lms
+
+
+def Lpt2XYZ(L, p, t, whitepoint=None, scale=1.0):
+	"""
+	Convert from Lpt to XYZ
+	
+	This is a modern update to L*a*b*, based on IPT space.
+	
+	Differences to L*a*b* and IPT:
+	- Using inverse CIE 2012 2degree LMS to XYZ matrix instead of
+	  Hunt-Pointer-Estevez Von Kries chromatic adapation in LMS space.
+	- Using L* compression rather than IPT pure 0.43 power.
+	- Tweaked LMS' to IPT matrix to account for change in XYZ to LMS matrix.
+	- Output scaled to L*a*b* type ranges, to maintain 1 JND scale.
+	- L* value is not a non-linear Y value.
+	
+	The input L* value needs to be in the nominal range [0.0, 100.0] and 
+	other input values scaled accordingly.
+	The output XYZ values are in the nominal range [0.0, 1.0].
+	
+	whitepoint can be string (e.g. "D50"), a tuple of XYZ coordinates or 
+	color temperature as float or int. Defaults to D50 if not set.
+	
+	"""
+	# Adapted from Argyll/icc/icc.c
+
+	xyz2lms = get_cat_matrix("CIE2012_2")
+	lms2xyz = xyz2lms.inverted()
+
+	wlms = xyz2lms * get_whitepoint(whitepoint, scale)
+
+	lms = Lpt2LMS_matrix * (L, p, t)
+
+	for j in xrange(3):
+		lms[j] = (lms[j] + 16.0) / 116.0
+
+		if lms[j] > 24.0 / 116.0:
+			lms[j] = pow(lms[j], 3.0)
+		else:
+			lms[j] = (lms[j] - 16.0 / 116.0) / 7.787036979
+
+		lms[j] *= wlms[j]
+
+	return lms2xyz * lms
+
+
 def XYZ2Lu_v_(X, Y, Z, whitepoint=None):
 	""" Convert from XYZ to CIE Lu'v' """
 
@@ -1553,7 +1956,8 @@ def XYZ2Luv(X, Y, Z, whitepoint=None):
 	return L, u, v
 
 
-def XYZ2RGB(X, Y, Z, rgb_space=None, scale=1.0, round_=False, clamp=True):
+def XYZ2RGB(X, Y, Z, rgb_space=None, scale=1.0, round_=False, clamp=True,
+			oetf=None):
 	"""
 	Convert from XYZ to RGB.
 	
@@ -1597,7 +2001,9 @@ def XYZ2RGB(X, Y, Z, rgb_space=None, scale=1.0, round_=False, clamp=True):
 			gamma = trc[i]
 		else:
 			gamma = trc
-		if isinstance(gamma, (list, tuple)):
+		if oetf:
+			RGB[i] = oetf(v)
+		elif isinstance(gamma, (list, tuple)):
 			RGB[i] = interp(v, gamma, [n / float(len(gamma) - 1) for n in
 									   xrange(len(gamma))])
 		else:
@@ -1627,8 +2033,8 @@ def XYZ2xyY(X, Y, Z, whitepoint=None):
 		# to 0, thus resulting in ZeroDivisionError later
 		x, y, Y = XYZ2xyY(*get_whitepoint(whitepoint))
 		return x, y, 0.0
-	x = X / (X + Y + Z)
-	y = Y / (X + Y + Z)
+	x = X / float(X + Y + Z)
+	y = Y / float(X + Y + Z)
 	return x, y, Y
 
 
@@ -2160,18 +2566,32 @@ class gam_fits(object):
 
 class Interp(object):
 
-	def __init__(self, xp, fp, left=None, right=None):
+	def __init__(self, xp, fp, left=None, right=None, use_numpy=False):
+		if use_numpy:
+			# Use numpy for speed
+			import numpy
+			xp = numpy.array(xp)
+			fp = numpy.array(fp)
+			self.numpy = numpy
 		self.xp = xp
 		self.fp = fp
 		self.left = left
 		self.right = right
 		self.lookup = {}
+		self.use_numpy = use_numpy
 
 	def __call__(self, x):
 		if not x in self.lookup:
-			self.lookup[x] = interp(x, self.xp, self.fp, self.left, self.right)
+			self.lookup[x] = self._interp(x)
 		return self.lookup[x]
 
+	def _interp(self, x):
+		if self.use_numpy:
+			return self.numpy.interp(x, self.xp, self.fp, self.left, self.right)
+		else:
+			return interp(x, self.xp, self.fp, self.left, self.right)
+		
+
 
 class BT1886(object):
 	# Adapted from ArgyllCMS xicc/xicc.c
@@ -2294,6 +2714,122 @@ class BT1886(object):
 		return out
 
 
+class BT2390(object):
+
+	"""
+	Roll-off for SMPTE 2084 (PQ) according to Report ITU-R BT.2390-2 HDR TV
+	
+	"""
+
+	def __init__(self, black_cdm2, white_cdm2, master_black_cdm2=0,
+				 master_white_cdm2=10000):
+		"""
+		Master black and white level are used to tweak the roll-off and clip.
+		
+		"""
+
+		self.black_cdm2 = black_cdm2
+		self.white_cdm2 = white_cdm2
+		self.master_black_cdm2 = master_black_cdm2
+		self.master_white_cdm2 = master_white_cdm2
+	
+		self.ominv = black_cdm2 / 10000.0  # Lmin
+		self.omini = specialpow(self.ominv, 1.0 / -2084)  # Original minLum
+		self.omaxv = white_cdm2 / 10000.0  # Lmax
+		self.omaxi = specialpow(self.omaxv, 1.0 / -2084)  # Original maxLum
+
+		self.oKS = 1.5 * self.omaxi - 0.5
+
+		# BT.2390-2
+		self.mminv = master_black_cdm2 / 10000.0  # LB
+		self.mmini = specialpow(self.mminv, 1.0 / -2084)
+		self.mmaxv = master_white_cdm2 / 10000.0  # LW
+		self.mmaxi = specialpow(self.mmaxv, 1.0 / -2084)
+		self.mini = (self.omini - self.mmini) / (self.mmaxi - self.mmini)  # Normalized minLum
+		self.minv = specialpow(self.mini, -2084)
+		self.maxi = (self.omaxi - self.mmini) / (self.mmaxi - self.mmini)  # Normalized maxLum
+		self.maxv = specialpow(self.maxi, -2084)
+
+		self.KS = 1.5 * self.maxi - 0.5
+
+	@staticmethod
+	def P(B, KS, maxi, maxci=1.0):
+		T = (B - KS) / (1 - KS)
+		E2 = ((2 * T ** 3 - 3 * T ** 2 + 1) * KS + (T ** 3 - 2 * T ** 2 + T) *
+			  (1 - KS) + (-2 * T ** 3 + 3 * T ** 2) * maxi)
+		if maxci < 1:
+			# Clipping for slightly better target display peak luminance usage
+			s = min(((B - KS) / (maxci - KS)) ** 4, 1.0)
+			E2 = E2 * (1 - s) + maxi * s
+		return E2
+
+	def apply(self, v, KS=None, maxi=None, maxci=1.0, mini=None,
+			  mmaxi=None, mmini=None, bpc=False, normalize=True):
+		"""
+		Apply roll-off (E' in, E' out)
+		maxci if < 1.0 applies alterante clip. SHOULD NOT BE USED FOR PQ.
+		
+		"""
+		if KS is None:
+			KS = self.KS
+		if maxi is None:
+			maxi = self.maxi
+		if mini is None:
+			mini = self.mini
+		if mmaxi is None:
+			mmaxi = self.mmaxi
+		if mmini is None:
+			mmini = self.mmini
+		if normalize and mmini is not None and mmaxi is not None:
+			# Normalize PQ values based on mastering display black/white levels
+			E1 = min(max((v - mmini) / (mmaxi - mmini), 0), 1.0)
+		else:
+			E1 = v
+		# BT.2390-3 suggests P[E1] if KS <= E1 <=1, but this results in
+		# division by zero if KS = 1. The correct way is to check for
+		# KS < E1 <=1
+		if KS < E1 <= 1:
+			E2 = self.P(E1, KS, maxi, maxci)
+		else:
+			E2 = E1
+		# BT.2390-3 suggests 0 <= E2 <= 1, but this results in a discontinuity
+		# if KS < 0 (high LB > Lmin, low Lmax, high LW). To avoid this, check
+		# for E2 <= 1 instead
+		if mini and E2 <= 1:
+			# Apply black level lift
+			minLum = mini
+			maxLum = maxi
+			b = minLum
+			# BT.2390-3 suggests E2 + b * (1 - E2) ** 4, but this clips, if
+			# minLum > 0.25, due to a 'dip' in the function. The solution is to
+			# adjust the exponent according to minLum. For minLum <= 0.25
+			# (< 5.15 cd/m2), this will give the same result as 'pure' BT.2390-3
+			if b >= 0:
+				# Only for positive b i.e. minLum >= LB
+				p = min(1.0 / b, 4)
+			else:
+				# For negative b i.e. minLum < LB
+				p = 4
+			E3 = E2 + b * (1 - E2) ** p
+			# If maxLum < 1, and the input value reaches maxLum, the resulting
+			# output value will be higher than maxLum after applying the black
+			# level lift (note that this is *not* a side-effect of the above 
+			# exponent adjustment). Undo this by re-scaling to the nominal output
+			# range [minLum, maxLum].
+			if maxi < 1:
+				# Only re-scale if maxLum < 1. Note that maxLum can be > 1
+				# if Lmax > LW despite E2 <= 1
+				E3 = convert_range(E3, b, maxi + b * (1 - maxi) ** p, b, maxi)
+		else:
+			E3 = E2
+		if bpc:
+			E3 = convert_range(E3, mini, maxi, 0, maxi)
+		if normalize and mmini is not None and mmaxi is not None:
+			# Invert the normalization of the PQ values
+			E3 = E3 * (mmaxi - mmini) + mmini
+		return max(E3, 0)
+
+
 class Matrix3x3(list):
 	
 	""" Simple 3x3 matrix """
@@ -2468,7 +3004,35 @@ cat_matrices = {"Bradford": Matrix3x3([[ 0.89510,  0.26640, -0.16140],
 										[ 0.00000,  0.00000,  0.91822]]),
 				"XYZ scaling": Matrix3x3([[1, 0, 0],
 										  [0, 1, 0],
-										  [0, 0, 1]])}
+										  [0, 0, 1]]),
+				"IPT":  Matrix3x3([[ 0.4002, 0.7075, -0.0807],
+								   [-0.2280, 1.1500,  0.0612],
+								   [ 0.0000, 0.0000,  0.9184]]),
+				# Inverse CIE 2012 2deg LMS to XYZ matrix from Argyll/icc/icc.c
+				"CIE2012_2": Matrix3x3([[ 0.2052445519046028,  0.8334486497310412, -0.0386932016356441],
+										[-0.4972221301804286,  1.4034846060306130,  0.0937375241498157],
+										[ 0.0000000000000000,  0.0000000000000000,  1.0000000000000000]])}
+
+LMS2IPT_matrix = Matrix3x3([[ 0.4000,  0.4000,  0.2000],
+							[ 4.4550, -4.8510,  0.3960],
+							[ 0.8056,  0.3572, -1.1628]])
+IPT2LMS_matrix = LMS2IPT_matrix.inverted()
+
+LinearRGB2LMS_matrix = Matrix3x3([[1688 / 4096., 2146 / 4096., 262 / 4096.],
+								  [683 / 4096., 2951 / 4096., 462 / 4096.],
+								  [99 / 4096., 309 / 4096., 3688 / 4096.]])
+LMS2LinearRGB_matrix = LinearRGB2LMS_matrix.inverted()
+L_M_S_2ICtCp_matrix = Matrix3x3([[.5, .5, 0],
+								 [6610 / 4096., -13613 / 4096., 7003 / 4096.],
+								 [17933 / 4096., -17390 / 4096., -543 / 4096.]])
+ICtCp2L_M_S__matrix = L_M_S_2ICtCp_matrix.inverted()
+
+# Tweaked LMS to IPT matrix to account for CIE 2012 2deg XYZ to LMS matrix
+# From Argyll/icc/icc.c
+LMS2Lpt_matrix = Matrix3x3([[ 0.6585034777870502,  0.1424555300344579,  0.1990409921784920],
+							[ 5.6413505933276049, -6.1697985811414187,  0.5284479878138138],
+							[ 1.6370552576322106,  0.0192823194340315, -1.6563375770662419]])
+Lpt2LMS_matrix = LMS2Lpt_matrix.inverted()
 
 standard_illuminants = {
 	# 1st level is the standard name => illuminant definitions
