@@ -325,6 +325,9 @@ def getbitmap(name, display_missing_icon=True, scale=True):
 					break
 			if path:
 				bitmaps[name] = wx.Bitmap(path)
+				if not bitmaps[name].IsOk():
+					path = None
+			if path:
 				if scale > 1 and i:
 					rescale = False
 					if i in (1, 2):
@@ -440,9 +443,11 @@ def get_display_number(display_no):
 	except IndexError:
 		return 0
 	else:
+		if display.endswith(" [PRIMARY]"):
+			display = " ".join(display.split(" ")[:-1])
 		for i in xrange(wx.Display.GetCount()):
 			geometry = "%i, %i, %ix%i" % tuple(wx.Display(i).Geometry)
-			if display.find("@ " + geometry) > -1:
+			if display.endswith("@ " + geometry):
 				if debug:
 					from log import safe_print
 					safe_print("[D] Found display %s at index %i" % 
@@ -468,10 +473,13 @@ def get_icon_bundle(sizes, name):
 	iconbundle = wx.IconBundle()
 	if not sizes:
 		# Assume ICO format
-		ico = get_data_path("theme/icons/%s.ico" % name)
-		if ico:
-			iconbundle.AddIconFromFile(ico, wx.BITMAP_TYPE_ICO)
-			return iconbundle
+		pth = get_data_path("theme/icons/%s.ico" % name)
+		if pth:
+			ico = wx.Icon(pth)
+			if ico.IsOk():
+				iconbundle.AddIcon(ico)
+				return iconbundle
+		sizes = [16]
 	for size in sizes:
 		iconbundle.AddIcon(get_bitmap_as_icon(size, name, False))
 	return iconbundle
@@ -633,7 +641,9 @@ cfg = ConfigParser.RawConfigParser()
 cfg.optionxform = str
 
 valid_ranges = {
-	"3dlut.hdr_peak_luminance": [120, 10000],
+	"3dlut.hdr_peak_luminance": [120.0, 10000.0],
+	"3dlut.hdr_minmll": [0.0, 0.1],
+	"3dlut.hdr_maxmll": [120.0, 10000.0],
 	"3dlut.trc_gamma": [0.000001, 10],
 	"3dlut.trc_output_offset": [0.0, 1.0],
 	"app.port": [1, 65535],
@@ -682,7 +692,7 @@ valid_values = {
 	"3dlut.image.order": ["rgb", "bgr"],
 	"3dlut.rendering_intent": intents,
 	"3dlut.size": [5, 9, 16, 17, 24, 32, 33, 64, 65],
-	"3dlut.trc": ["bt1886", "customgamma", "gamma2.2", "smpte2084.hardclip", "smpte2084.rolloffclip"],
+	"3dlut.trc": ["bt1886", "customgamma", "gamma2.2", "smpte2084.hardclip", "smpte2084.rolloffclip", "hlg"],
 	"3dlut.trc_gamma_type": ["b", "B"],
 	"calibration.quality": ["v", "l", "m", "h", "u"],
 	"colorimeter_correction.observer": observers,
@@ -768,8 +778,10 @@ defaults = {
 	"3dlut.format": "cube",
 	"3dlut.gamap.use_b2a": 0,
 	"3dlut.hdr_display": 0,
-	"3dlut.hdr_maxcll": 10000,
-	"3dlut.hdr_peak_luminance": 400,
+	"3dlut.hdr_minmll": 0.0,
+	"3dlut.hdr_maxmll": 10000.0,
+	"3dlut.hdr_peak_luminance": 400.0,
+	"3dlut.hdr_ambient_luminance": 5.0,
 	"3dlut.image.layout": "h",
 	"3dlut.image.order": "rgb",
 	"3dlut.input.profile": "",
@@ -1004,6 +1016,7 @@ defaults = {
 	"profile_loader.known_window_classes": ";".join(["CalClient.exe"]),
 	"profile_loader.quantize_bits": 16,
 	"profile_loader.reset_gamma_ramps": 0,
+	"profile_loader.show_notifications": 0,
 	"profile_loader.track_other_processes": 1,
 	"profile_loader.use_madhcnet": 0,
 	"profile_loader.verify_calibration": 0,
@@ -1305,7 +1318,7 @@ def get_display_profile(display_no=None):
 	except Exception, exception:
 		from log import _safe_print, log
 		_safe_print("ICCP.get_display_profile(%s):" % display_no, 
-					exception, fn=log)
+					safe_unicode(exception), fn=log)
 
 
 standard_profiles = []
@@ -1703,3 +1716,9 @@ def writecfg(which="user", worker=None, module=None, options=()):
 
 _init_testcharts()
 runtype = runtimeconfig(pyfile)
+
+if isapp and not os.getenv("SSL_CERT_FILE"):
+	# Use our bundled CA file
+	cafile = get_data_path("cacert.pem")
+	if cafile:
+		os.environ["SSL_CERT_FILE"] = cafile.encode(fs_enc, "replace")
