@@ -7,8 +7,9 @@ import subprocess as sp
 import sys
 import tempfile
 
+import numpy
+
 from argyll_cgats import cal_to_fake_profile, vcgt_to_cal
-from colormath import interp
 from config import (fs_enc, get_argyll_display_number, get_data_path,
 					get_display_profile, get_display_rects, getcfg, geticon,
 					get_verified_path, setcfg)
@@ -24,7 +25,7 @@ from wxaddons import get_platform_window_decoration_size, wx
 from wxMeasureFrame import MeasureFrame
 from wxwindows import (BaseApp, BaseFrame, BitmapBackgroundPanelText,
 					   CustomCheckBox, FileDrop, InfoDialog, TooltipWindow)
-from wxfixes import GenBitmapButton as BitmapButton
+from wxfixes import GenBitmapButton as BitmapButton, wx_Panel
 import colormath
 import config
 import wxenhancedplot as plot
@@ -80,7 +81,8 @@ class CoordinateType(list):
 				vmax = self[-1][0]
 		return colormath.get_gamma(values, 100.0, vmin, vmax, average, least_squares)
 	
-	def get_transfer_function(self, best=True, slice=(0.05, 0.95)):
+	def get_transfer_function(self, best=True, slice=(0.05, 0.95),
+							  outoffset=None):
 		"""
 		Return transfer function name, exponent and match percentage
 		
@@ -88,66 +90,18 @@ class CoordinateType(list):
 		transfer_function = self._transfer_function.get((best, slice))
 		if transfer_function:
 			return transfer_function
-		trc = ICCP.CurveType()
-		match = {}
-		vmin = self[0][0]
-		vmax = self[-1][0]
-		best_yx = (0, 255)
-		for i, (y, x) in enumerate(self):
-			if x - 127.5 > 0 and x < best_yx[1]:
-				best_yx = (y, x)
-		gamma = colormath.get_gamma([(best_yx[1] / 255.0 * 100.0, best_yx[0])], 100.0, vmin, vmax)
-		for name, exp in (("Rec. 709", -709),
-						  ("Rec. 1886", -1886),
-						  ("SMPTE 240M", -240),
-						  ("SMPTE 2084", -2084),
-						  ("DICOM", -1023),
-						  ("L*", -3.0),
-						  ("sRGB", -2.4),
-						  ("Gamma %.2f" % gamma, gamma)):
-			if name in ("DICOM", "Rec. 1886", "SMPTE 2084"):
-				if self.profile and isinstance(self.profile.tags.get("lumi"),
-											   ICCP.XYZType):
-					white_cdm2 = self.profile.tags.lumi.Y
-				else:
-					white_cdm2 = 100.0
-				black_Y = vmin / 100.0
-				black_cdm2 = black_Y * white_cdm2
-				try:
-					if name == "DICOM":
-						trc.set_dicom_trc(black_cdm2, white_cdm2, size=len(self))
-					elif name == "Rec. 1886":
-						trc.set_bt1886_trc(black_Y, size=len(self))
-					elif name == "SMPTE 2084":
-						trc.set_smpte2084_trc(black_cdm2, white_cdm2, size=len(self))
-				except ValueError:
-					continue
-			else:
-				trc.set_trc(exp, len(self), vmin * 655.35, vmax * 655.35)
-			match[(name, exp)] = 0.0
-			count = 0
-			start = slice[0] * 255
-			end = slice[1] * 255
-			for i, (n, x) in enumerate(self):
-				##n = colormath.XYZ2Lab(0, n, 0)[0]
-				if x >= start and x <= end:
-					n = colormath.get_gamma([(x / 255.0 * 100, n)], 100.0, vmin, vmax, False)
-					if n:
-						n = n[0]
-						##n2 = colormath.XYZ2Lab(0, trc[i][0], 0)[0]
-						n2 = colormath.get_gamma([(i / (len(trc) - 1.0) * 65535, trc[i])], 65535, vmin * 655.35, vmax * 655.35, False)
-						if n2:
-							n2 = n2[0]
-							match[(name, exp)] += 1 - (max(n, n2) - min(n, n2)) / n2
-							count += 1
-			if count:
-				match[(name, exp)] /= count
-		if not best:
-			self._transfer_function[(best, slice)] = match
-			return match
-		match, (name, exp) = sorted(zip(match.values(), match.keys()))[-1]
-		self._transfer_function[(best, slice)] = (name, exp), match
-		return (name, exp), match
+		xp = []
+		fp = []
+		for y, x in self:
+			xp.append(x)
+			fp.append(y)
+		interp = colormath.Interp(xp, fp, use_numpy=True)
+		otrc = ICCP.CurveType(profile=self.profile)
+		for i in xrange(len(self)):
+			otrc.append(interp(i / (len(self) - 1.0) * 255) / 100 * 65535)
+		match = otrc.get_transfer_function(best, slice, outoffset=outoffset)
+		self._transfer_function[(best, slice)] = match
+		return match
 	
 	def set_trc(self, power=2.2, values=(), vmin=0, vmax=100):
 		"""
@@ -227,6 +181,8 @@ class LUTCanvas(plot.PlotCanvas):
 			axis_x = 100.0
 		else:
 			axis_x = 255.0
+		if getattr(self, "axis_x", None) != (0, axis_x):
+			wx.CallAfter(self.center)
 		self.axis_x, self.axis_y = (0, axis_x), (0, axis_y)
 		if not self.last_draw:
 			self.center_x = axis_x / 2.0
@@ -731,7 +687,7 @@ class LUTFrame(BaseFrame):
 		self.sizer = wx.BoxSizer(wx.VERTICAL)
 		self.SetSizer(self.sizer)
 
-		panel = wx.Panel(self)
+		panel = wx_Panel(self)
 		panel.SetBackgroundColour(BGCOLOUR)
 		self.sizer.Add(panel, flag=wx.EXPAND)
 		panel.SetSizer(wx.FlexGridSizer(0, 5, 0, 8))
@@ -780,11 +736,11 @@ class LUTFrame(BaseFrame):
 		self.client = LUTCanvas(self)
 		self.sizer.Add(self.client, 1, wx.EXPAND)
 		
-		self.box_panel = wx.Panel(self)
+		self.box_panel = wx_Panel(self)
 		self.box_panel.SetBackgroundColour(BGCOLOUR)
 		self.sizer.Add(self.box_panel, flag=wx.EXPAND)
 		
-		self.status = BitmapBackgroundPanelText(self)
+		self.status = BitmapBackgroundPanelText(self, name="statuspanel")
 		self.status.SetMaxFontSize(11)
 		self.status.label_y = 8
 		self.status.textshadow = False
@@ -1165,8 +1121,13 @@ class LUTFrame(BaseFrame):
 	
 	def lookup_tone_response_curves(self, intent="r"):
 		""" Lookup Y -> RGB tone values through TRC tags or LUT """
-		
-		mult = 2
+
+		profile = self.profile
+
+		if profile.connectionColorSpace == "RGB":
+			mult = 1
+		else:
+			mult = 4
 		size = 256 * mult  # Final number of coordinates
 
 		if hasattr(self, "rendering_intent_select"):
@@ -1186,15 +1147,13 @@ class LUTFrame(BaseFrame):
 						len(self.bTRC))
 		has_same_trc = self.rTRC == self.gTRC == self.bTRC
 
-		profile = self.profile
-
 		if profile.version >= 4:
 			self.client.errors.append(Error("\n".join([lang.getstr("profile.iccv4.unsupported"),
 													   profile.getDescription()])))
 			return
 
 		if (profile.colorSpace not in ("RGB", "GRAY") or
-			profile.connectionColorSpace not in ("Lab", "XYZ")):
+			profile.connectionColorSpace not in ("Lab", "XYZ", "RGB")):
 			if profile.colorSpace not in ("RGB", "GRAY"):
 				unsupported_colorspace = profile.colorSpace
 			else:
@@ -1206,6 +1165,8 @@ class LUTFrame(BaseFrame):
 		
 		if profile.colorSpace == "GRAY":
 			direction = "b"
+		elif profile.connectionColorSpace == "RGB":
+			direction = "f"
 		elif "B2A0" in profile.tags:
 			direction = {0: "b",
 						 1: "if",
@@ -1237,6 +1198,10 @@ class LUTFrame(BaseFrame):
 			pcs = "x"
 			for Lab in Lab_triplets:
 				XYZ_triplets.append(colormath.Lab2XYZ(*Lab))
+		elif profile.connectionColorSpace == "RGB":
+			use_icclu = False
+			pcs = None
+			intent = None
 		else:
 			use_icclu = False
 			pcs = "l"
@@ -1257,14 +1222,37 @@ class LUTFrame(BaseFrame):
 		try:
 			odata = self.worker.xicclu(profile, idata, intent,
 									   direction, order, pcs,
-									   use_icclu=use_icclu)
+									   use_icclu=use_icclu,
+									   get_clip=direction == "if")
 		except Exception, exception:
 			self.client.errors.append(Error(safe_unicode(exception)))
 		
 		if self.client.errors:
 			return
 
-		if direction in ("b", "if"):
+		if direction in ("b", "if") or profile.connectionColorSpace == "RGB":
+			if direction == "if":
+				Lbp = self.worker.xicclu(profile, [[0, 0, 0]], intent, "f",
+										 order, "l", use_icclu=use_icclu)[0][0]
+				maxval = size - 1.0
+
+				# Deal with values that got clipped (below black as well as white)
+				do_low_clip = True
+				for i, values in enumerate(odata):
+					if values[3] is True or i == 0:
+						if do_low_clip and (i / maxval * 100 < Lbp or i == 0):
+							# Set to black
+							values[:] = [0.0, 0.0, 0.0]
+						elif (i == maxval and
+							  [round(v, 4) for v in values[:3]] == [1, 1, 1]):
+							# Set to white
+							values[:] = [1.0, 1.0, 1.0]
+					else:
+						# First non-clipping value disables low clipping
+						do_low_clip = False
+					if len(values) > 3:
+						values.pop()
+
 			RGB_triplets = odata
 		else:
 			Lab_triplets = odata
@@ -1279,18 +1267,32 @@ class LUTFrame(BaseFrame):
 					not min(RGB_triplets[j + 1][i], 1.0)):
 					continue
 				v *= 255
-				X, Y, Z = colormath.Lab2XYZ(*Lab_triplets[j], **{"scale": 100})
-				if direction in ("b", "if"):
-					X = Z = Y
-				elif intent == "a":
-					wp = profile.tags.wtpt.ir.values()
-					X, Y, Z = colormath.adapt(X, Y, Z, wp, (1, 1, 1))
-				if i == 0:
-					self.rTRC.append([X, v])
-				elif i == 1:
-					self.gTRC.append([Y, v])
-				elif i == 2:
-					self.bTRC.append([Z, v])
+				if profile.connectionColorSpace == "RGB":
+					x = j / (size - 1.0) * 255
+					if i == 0:
+						self.rTRC.append([x, v])
+					elif i == 1:
+						self.gTRC.append([x, v])
+					elif i == 2:
+						self.bTRC.append([x, v])
+				else:
+					X, Y, Z = colormath.Lab2XYZ(*Lab_triplets[j], **{"scale": 100})
+					if direction in ("b", "if"):
+						X = Z = Y
+					elif intent == "a":
+						wp = profile.tags.wtpt.ir.values()
+						X, Y, Z = colormath.adapt(X, Y, Z, wp, (1, 1, 1))
+					elif intent != "a":
+						wp = profile.tags.wtpt.ir.values()
+						X, Y, Z = colormath.adapt(X, Y, Z, "D50", (1, 1, 1))
+					if i == 0:
+						self.rTRC.append([X, v])
+					elif i == 1:
+						self.gTRC.append([Y, v])
+					elif i == 2:
+						self.bTRC.append([Z, v])
+		if profile.connectionColorSpace == "RGB":
+			return
 		if use_trc_tags:
 			if has_same_trc:
 				self.bTRC = self.gTRC = self.rTRC
@@ -1308,8 +1310,8 @@ class LUTFrame(BaseFrame):
 			for i in xrange(size):
 				x.append(i / (size - 1.0) * 255)
 				y.append(colormath.Lab2XYZ(i / (size - 1.0) * 100, 0, 0)[1] * 100)
-			xi = interp(y, yp, xp)
-			yi = interp(x, xi, y)
+			xi = numpy.interp(y, yp, xp)
+			yi = numpy.interp(x, xi, y)
 			setattr(self, "tf_" + sig, CoordinateType(self.profile))
 			for Y, v in zip(yi, x):
 				if Y <= yp[0]:
@@ -1332,9 +1334,8 @@ class LUTFrame(BaseFrame):
 		event.Skip()
 	
 	def plot_mode_select_handler(self, event):
-		self.client.resetzoom()
+		#self.client.resetzoom()
 		self.DrawLUT()
-		wx.CallAfter(self.client.center)
 
 	def get_commands(self):
 		return self.get_common_commands() + ["curve-viewer [filename]",
@@ -1414,6 +1415,25 @@ class LUTFrame(BaseFrame):
 				wx.CallAfter(show_result_dialog, exception, self)
 			else:
 				curves.append(lang.getstr('[rgb]TRC'))
+		if getcfg("show_advanced_options"):
+			if isinstance(self.profile.tags.get("A2B0"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.A2B0.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.A2B0.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("A2B1"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.A2B1.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.A2B1.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("A2B2"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.A2B2.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.A2B2.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A0"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.B2A0.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.B2A0.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A1"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.B2A1.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.B2A1.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A2"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.B2A2.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.B2A2.shaper_curves.output'))
 		selection = self.plot_mode_select.GetSelection()
 		center = False
 		if curves and (selection < 0 or selection > len(curves) - 1):
@@ -1437,7 +1457,7 @@ class LUTFrame(BaseFrame):
 			'vcgt' in self.profile.tags):
 			if 'R' in colorants or 'G' in colorants or 'B' in colorants:
 				legend.append(lang.getstr("tone_values"))
-				if '=' in colorants:
+				if '=' in colorants and 0:  # NEVER
 					unique = []
 					if 'R' in colorants:
 						unique.append(self.client.r_unique)
@@ -1506,7 +1526,8 @@ class LUTFrame(BaseFrame):
 										  self.tf_gTRC[i][1] +
 										  self.tf_bTRC[i][1]) / 3.0])
 			if getattr(self, "trc", None):
-				transfer_function = self.trc.get_transfer_function(slice=(0.00, 1.00))
+				transfer_function = self.trc.get_transfer_function(slice=(0.00, 1.00),
+																   outoffset=1.0)
 			#if "R" in colorants and "G" in colorants and "B" in colorants:
 				#if self.profile.tags.rTRC == self.profile.tags.gTRC == self.profile.tags.bTRC:
 					#transfer_function = self.profile.tags.rTRC.get_transfer_function()
@@ -1546,6 +1567,14 @@ class LUTFrame(BaseFrame):
 		curves = None
 		if self.profile:
 			if self.plot_mode_select.GetSelection() == 0:
+				# Convert calibration information from embedded WCS profile
+				# (if present) to VideCardFormulaType if the latter is not present
+				if (isinstance(self.profile.tags.get("MS00"),
+							   ICCP.WcsProfilesTagType) and
+					not "vcgt" in self.profile.tags):
+					vcgt = self.profile.tags["MS00"].get_vcgt()
+					if vcgt:
+						self.profile.tags["vcgt"] = vcgt
 				if 'vcgt' in self.profile.tags:
 					curves = self.profile.tags['vcgt']
 				else:
@@ -1578,6 +1607,40 @@ class LUTFrame(BaseFrame):
 						'entryCount': len(self.rTRC),
 						'entrySize': 2
 					}
+			elif self.plot_mode_select.GetSelection() in range(2, 14):
+				if self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B0.shaper_curves.input'):
+					tables = self.profile.tags.A2B0.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B0.shaper_curves.output'):
+					tables = self.profile.tags.A2B0.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B1.shaper_curves.input'):
+					tables = self.profile.tags.A2B1.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B1.shaper_curves.output'):
+					tables = self.profile.tags.A2B1.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B2.shaper_curves.input'):
+					tables = self.profile.tags.A2B2.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B2.shaper_curves.output'):
+					tables = self.profile.tags.A2B2.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A0.shaper_curves.input'):
+					tables = self.profile.tags.B2A0.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A0.shaper_curves.output'):
+					tables = self.profile.tags.B2A0.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A1.shaper_curves.input'):
+					tables = self.profile.tags.B2A1.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A1.shaper_curves.output'):
+					tables = self.profile.tags.B2A1.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A2.shaper_curves.input'):
+					tables = self.profile.tags.B2A2.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A2.shaper_curves.output'):
+					tables = self.profile.tags.B2A2.output
+				entry_count = len(tables[0])
+				x = [v / (entry_count - 1.0) * 255 for v in range(entry_count)]
+				curves = {
+					'data': [zip(x, [v / 65535.0 * 255 for v in tables[0]]),
+							 zip(x, [v / 65535.0 * 255 for v in tables[1]]),
+							 zip(x, [v / 65535.0 * 255 for v in tables[2]])],
+					'entryCount': entry_count,
+					'entrySize': 2
+				}
 		yLabel = []
 		if self.toggle_red.GetValue():
 			yLabel.append("R")
@@ -1585,7 +1648,8 @@ class LUTFrame(BaseFrame):
 			yLabel.append("G")
 		if self.toggle_blue.GetValue():
 			yLabel.append("B")
-		if self.plot_mode_select.GetSelection() == 0:
+		if (self.plot_mode_select.GetSelection() in [0] + range(2, 14) or
+			(self.profile and self.profile.connectionColorSpace == "RGB")):
 			self.xLabel = "".join(yLabel)
 		else:
 			if self.show_as_L.GetValue():
@@ -1598,11 +1662,14 @@ class LUTFrame(BaseFrame):
 		self.toggle_green.Enable(bool(curves))
 		self.toggle_blue.Enable(bool(curves))
 		self.show_as_L.Enable(bool(curves))
-		self.show_as_L.Show(self.plot_mode_select.GetSelection() != 0)
-		self.toggle_clut.Show(self.plot_mode_select.GetSelection() == 1 and
+		self.show_as_L.Show(self.plot_mode_select.GetSelection() not in [0] + range(2, 14) and
+							self.profile.connectionColorSpace != "RGB")
+		self.toggle_clut.Show(self.profile.connectionColorSpace != "RGB" and
+							  self.plot_mode_select.GetSelection() == 1 and
 							  ("B2A0" in self.profile.tags or
 							   "A2B0" in self.profile.tags))
-		self.toggle_clut.Enable(self.plot_mode_select.GetSelection() == 1 and
+		self.toggle_clut.Enable(self.profile.connectionColorSpace != "RGB" and
+								self.plot_mode_select.GetSelection() == 1 and
 								isinstance(self.profile.tags.get("rTRC"),
 										   ICCP.CurveType) and
 								isinstance(self.profile.tags.get("gTRC"),
@@ -1638,7 +1705,8 @@ class LUTFrame(BaseFrame):
 		if hasattr(self, "show_actual_lut_cb"):
 			self.show_actual_lut_cb.Show(self.plot_mode_select.GetSelection() == 0)
 		if hasattr(self, "rendering_intent_select"):
-			self.rendering_intent_select.Show(self.plot_mode_select.GetSelection() == 1)
+			self.rendering_intent_select.Show(self.plot_mode_select.GetSelection() == 1 and
+											  self.profile.connectionColorSpace != "RGB")
 		if hasattr(self, "direction_select"):
 			self.direction_select.Show(self.toggle_clut.IsShown() and
 									   self.toggle_clut.GetValue() and
@@ -1668,7 +1736,10 @@ class LUTFrame(BaseFrame):
 		if self.worker.tempdir and os.path.isdir(self.worker.tempdir):
 			self.worker.wrapup(False)
 		config.writecfg(module="curve-viewer", options=("display.number", ))
-		event.Skip()
+		# Hide first (looks nicer)
+		self.Hide()
+		# Need to use CallAfter to prevent hang under Windows if minimized
+		wx.CallAfter(self.Destroy)
 
 	def OnMotion(self, event):
 		if isinstance(event, wx.MouseEvent):
@@ -1801,19 +1872,30 @@ class LUTFrame(BaseFrame):
 							0 if self.toggle_green.GetValue() else None),
 						   self.client.point_grid[2].get(pointXY[0],
 							0 if self.toggle_blue.GetValue() else None))
-				if (self.plot_mode_select.GetSelection() == 0 or
-					R == G == B or ((R == G or G == B or R == B) and
+				if (R == G == B or ((R == G or G == B or R == B) and
 									None in (R, G ,B))):
 					rgb = ""
 				else:
 					rgb = legend[0] + " "
-				if self.plot_mode_select.GetSelection() == 1:
+				if 1:
 					joiner = u" \u2192 "
-					if self.show_as_L.GetValue():
-						format = "L* %.4f", "%s %.2f"
+					if (self.plot_mode_select.GetSelection() == 1 and
+						self.profile.connectionColorSpace != "RGB"):
+						if self.show_as_L.GetValue():
+							format = "L* %.4f", "%s %.2f"
+						else:
+							format = "Y %.4f", "%s %.2f"
+						axis_y = 100.0
 					else:
-						format = "Y %.4f", "%s %.2f"
-					axis_y = 100.0
+						label = []
+						if R is not None:
+							label.append("R")
+						if G is not None:
+							label.append("G")
+						if B is not None:
+							label.append("B")
+						format = "%s %%.2f" % "=".join(label), "%s %.2f"
+						axis_y = 255.0
 					if R == G == B or ((R == G or G == B or R == B) and
 									   None in (R, G ,B)):
 						#if R is None:
@@ -1828,7 +1910,9 @@ class LUTFrame(BaseFrame):
 										filter(lambda v: v[1] is not None,
 											   (("R", R), ("G", G), ("B", B)))])
 					legend[0] = joiner.join([format[0] % pointXY[0], RGB])
-					pointXY = pointXY[1], pointXY[0]
+					if (self.plot_mode_select.GetSelection() == 1 and
+						self.profile.connectionColorSpace != "RGB"):
+						pointXY = pointXY[1], pointXY[0]
 				else:
 					joiner = u" \u2192 "
 					format = "%.2f", "%.2f"
@@ -1836,14 +1920,36 @@ class LUTFrame(BaseFrame):
 					legend[0] += " " + joiner.join([format[i] % point
 													for i, point in
 													enumerate(pointXY)])
-				if (len(legend) == 1 and pointXY[0] > 0 and
-					pointXY[0] < 255 and pointXY[1] > 0):
-					y = pointXY[1]
-					if (self.plot_mode_select.GetSelection() == 1 and
-						self.show_as_L.GetValue()):
-						y = colormath.Lab2XYZ(y, 0, 0)[1] * 100
-					legend.append(rgb + "Gamma %.2f" % (math.log(y / axis_y) / math.log(pointXY[0] / 255.0)))
-				self.add_tone_values(legend)
+				if len(legend) == 1:
+					gamma = []
+					for label, v in (("R", R), ("G", G), ("B", B)):
+						if v is None:
+							continue
+						if (self.plot_mode_select.GetSelection() == 1 and
+							self.profile.connectionColorSpace != "RGB"):
+							x = v
+							y = pointXY[1]
+							if self.show_as_L.GetValue():
+								y = colormath.Lab2XYZ(y, 0, 0)[1] * 100
+						else:
+							x = pointXY[0]
+							y = v
+						if x <= 0 or x >= 255 or y <= 0 or y >= axis_y:
+							continue
+						if R == G == B or ((R == G or G == B or R == B) and
+										   None in (R, G ,B)):
+							label = "=".join(["%s" % s for s, v in
+											  filter(lambda (s, v): v is not None,
+													 (("R", R), ("G", G), ("B", B)))])
+						# Note: We need to make sure each point is a float because it
+						# might be a decimal.Decimal, which can't be divided by floats!
+						gamma.append(label + " %.2f" % round(math.log(float(y) / axis_y) / math.log(x / 255.0), 2))
+						if "=" in label:
+							break
+					if gamma:
+						legend.append("Gamma " + " ".join(gamma))
+				if self.profile.connectionColorSpace != "RGB":
+					self.add_tone_values(legend)
 				legend = [", ".join(legend[:-1])] + [legend[-1]]
 				self.SetStatusText("\n".join(legend))
 				# Make up dictionary to pass to DrawPointLabel
@@ -1867,7 +1973,8 @@ class LUTFrame(BaseFrame):
 
 	def display_changed(self, event):
 		self.worker.enumerate_displays_and_ports(check_lut_access=False,
-												 enumerate_ports=False)
+												 enumerate_ports=False,
+												 include_network_devices=False)
 
 
 def main():
